import asyncio
import sys

import injection  # noqa
from api_app import APIService
from dependency import container
from example_service import ExampleService
from sample import Sample

from chromatrace import LoggingConfig, LoggingSettings, tracer

sys.stdout.reconfigure(encoding='utf-8')
app = container[APIService].app

@tracer
def main():
    # Optional: Set global log level
    container[LoggingSettings].log_level = "DEBUG"
    logger = container[LoggingConfig].get_logger("Main")

    logger.info("Starting main")
    service = container[ExampleService]
    sample = container[Sample]
    asyncio.run(service.do_something())
    sample.do_something()
    logger.info("Finished main, Run API Service")
    
    container[APIService].run()

if __name__ == "__main__":
    main()


from fastapi import Request
from starlette.middleware.base import BaseHTTPMiddleware

from .tracer import RequestIdContext, trace_id_ctx


class RequestIdMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        request_id = request.headers.get("X-Request-ID")
        with RequestIdContext(request_id):
            response = await call_next(request)
            response.headers["X-Request-ID"] = trace_id_ctx.get()
            return response


from example_service import ExampleService
from fastapi import FastAPI, Request
from fastapi.middleware.cors import CORSMiddleware
from starlette.middleware.base import BaseHTTPMiddleware

from chromatrace import LoggingConfig, RequestIdContext, trace_id_ctx


class APIService:
    def __init__(self, logging_config: LoggingConfig, example_service: ExampleService):
        self.logger = logging_config.get_logger(self.__class__.__name__)
        self.app = FastAPI()
        self.example_service = example_service
        # Add middleware
        self.app.add_middleware(
            CORSMiddleware,
            allow_origins=["*"],  # Allows all origins
            allow_credentials=True,
            allow_methods=["*"],  # Allows all methods
            allow_headers=["*"],  # Allows all headers
        )

        class RequestIdMiddleware(BaseHTTPMiddleware):
            async def dispatch(self, request: Request, call_next):
                request_id = request.headers.get("X-Request-ID")
                with RequestIdContext(request_id):
                    response = await call_next(request)
                    response.headers["X-Request-ID"] = trace_id_ctx.get()
                    return response

        self.app.add_middleware(RequestIdMiddleware)
        self.do_something()
        self.routes()
    
    def do_something(self):
        self.logger.debug("Check something in API service")
        self.logger.info("Doing something in API service")
        self.logger.error("Something went wrong in API service")

    def run(self):
        import uvicorn
        uvicorn.run(self.app, host="0.0.0.0", port=8000,)

    def routes(self):

        @self.app.get("/")
        async def read_root():
            self.logger.info("Hello World")
            await self.example_service.do_something()
            return {"message": "Hello World"}



import logging
from pathlib import Path
from typing import Optional

from pydantic import BaseModel


class LoggingSettings(BaseModel):
    log_level: str = "INFO"
    log_format: str = "(%(asctime)s)-[%(levelname)s]-[%(name)s]-FILENAME:%(filename)s-FUNC:%(funcName)s-THREAD:%(threadName)s-LINE:%(lineno)d :: \n%(message)s"
    date_format: str = "%Y-%m-%d %H:%M:%S"
    style: str = "%"
    file_path: Path = Path("./log/app.log")
    max_bytes: int = 500 * 1024 * 1024  # 500MB
    backup_count: int = 3  # 3 days retention
    syslog_host: Optional[str] = None
    syslog_port: Optional[int] = None


class ColoredFormatter(logging.Formatter):
    def __init__(self, fmt=None, datefmt=None, style="%"):
        super().__init__(fmt, datefmt, style)
        self._level_color_format = {
            logging.DEBUG: "\033[94m{}\033[0m",  # Blue
            logging.INFO: "\033[92m{}\033[0m",  # Green
            logging.WARNING: "\033[93m{}\033[0m",  # Yellow
            logging.ERROR: "\033[91m{}\033[0m",  # Red
            logging.CRITICAL: "\033[95m{}\033[0m",  # Magenta
        }
        self._message_color_format = "\033[1m{}\033[0m"  # Bold
        self._name_color_format = "\033[90m{}\033[0m"  # Gray
        self.splitter = "\n"  # To add a newline after each log message

    def format(self, record):
        record.levelname = self._level_color_format.get(record.levelno, "").format(
            record.levelname
        )
        record.msg = self._message_color_format.format(record.msg)
        record.name = self._name_color_format.format(record.name)
        return super(ColoredFormatter, self).format(record) + self.splitter

class IgnoreNANTraceFormatter(ColoredFormatter):
    def format(self, record):
        try:
            message = super().format(record)
            return message.replace("[NAN]-", "")
        except Exception:
            return record.getMessage()

class SysLogFormatter(ColoredFormatter):
    def format(self, record):
        try:
            message = super().format(record)
            return message.replace("\n", " | ")
        except Exception:
            return record.getMessage()


class ApplicationLevelFilter(logging.Filter):
    def __init__(self, application_level):
        super().__init__()
        self.application_level = application_level

    def filter(self, record):
        record.application_level = self.application_level
        return True






import logging
import logging.handlers
import queue
import socket

from .logging_settings import (ApplicationLevelFilter, ColoredFormatter,
                               IgnoreNANTraceFormatter, LoggingSettings,
                               SysLogFormatter)
from .tracer import RequestIdFilter


class LoggingConfig:
    def __init__(
            self, 
            settings: LoggingSettings, 
            application_level: str = '', 
            enable_tracing: bool = True,
            ignore_nan_trace: bool = False):
        self.settings = settings
        self._configured = False
        self.request_id_filter = RequestIdFilter()
        self.application_level = application_level
        if self.application_level:
            self.application_level_filter = ApplicationLevelFilter(self.application_level)
            self.settings.log_format = '[%(application_level)s]-' + self.settings.log_format
        self.enable_tracing = enable_tracing
        if self.enable_tracing:
            self.settings.log_format = '[%(trace_id)s]-' + self.settings.log_format
        self.ignore_nan_trace = ignore_nan_trace

    def configure(self):
        if self._configured:
            return

        # Set root logger level
        logging.getLogger().setLevel(self.settings.log_level)

    def get_logger(self, name: str) -> logging.Logger:
        if not self._configured:
            self.configure()

        logger = logging.getLogger(name)
        logger.propagate = False  # Prevent double logging

        # Add handlers if they don't exist
        if not logger.handlers:
            self._setup_handlers(logger)

        return logger

    def _set_logger_settings(self, handler, formatter, logger):
        handler.setFormatter(formatter)
        handler.addFilter(self.request_id_filter)
        if self.application_level:
            handler.addFilter(self.application_level_filter)
        logger.addHandler(handler)

    def _setup_handlers(self, logger: logging.Logger):
        formatter = ColoredFormatter(
            fmt=self.settings.log_format,
            datefmt=self.settings.date_format,
            style=self.settings.style,
        )
        if self.enable_tracing and self.ignore_nan_trace:
            formatter = IgnoreNANTraceFormatter(
                fmt=self.settings.log_format,
                datefmt=self.settings.date_format,
                style=self.settings.style,
            )

        # Queue handler
        log_queue = queue.Queue()

        # Console handler
        console_handler = logging.StreamHandler()
        self._set_logger_settings(console_handler, formatter, logger)

        # File handler with rotation
        file_handler = logging.handlers.RotatingFileHandler(
            filename=str(self.settings.file_path),
            maxBytes=self.settings.max_bytes,
            backupCount=self.settings.backup_count,
            encoding="utf-8",
        )
        self._set_logger_settings(file_handler, formatter, logger)
        

        # Setup syslog with error handling
        syslog_handler = self._setup_syslog_handler(logger, formatter)

        # Queue listener
        handler = [console_handler, file_handler, syslog_handler]
        queue_listener = logging.handlers.QueueListener(log_queue, *handler, respect_handler_level=True)
        queue_listener.start()


    def _setup_syslog_handler(self, logger, formatter):
        if not self.settings.syslog_host or not self.settings.syslog_port:
            return

        try:
            syslog_formatter = SysLogFormatter(
                fmt=self.settings.log_format,
                datefmt=self.settings.date_format,
                style=self.settings.style,
            )
            
            # Create handler with socket handling
            syslog_handler = logging.handlers.SysLogHandler(
                address=(self.settings.syslog_host, self.settings.syslog_port),
                facility=logging.handlers.SysLogHandler.LOG_USER,
                socktype=socket.SOCK_DGRAM
            )

            # Add error handler
            syslog_handler.handleError = lambda *args, **kwargs: None
            
            self._set_logger_settings(syslog_handler, syslog_formatter, logger)

            return syslog_handler
            

        except (socket.error, OSError) as e:
            # Fallback to console logging if syslog fails
            console = logging.StreamHandler()
            console.setFormatter(formatter)
            logger.addHandler(console)
            logger.warning(f"Failed to setup syslog handler: {str(e)}")
            print(f"Failed to setup syslog handler: {str(e)}")

    


import logging

from chromatrace import LoggingConfig


class Sample:
    def __init__(self, logging_config: LoggingConfig):
        self.logger = logging_config.get_logger(self.__class__.__name__)
        self.logger.setLevel(logging.DEBUG)
        
    def do_something(self):
        self.logger.debug("Check something")
        self.logger.info("Doing something")
        self.logger.warning("Doing something")
        self.logger.error("Something went wrong")


import logging

from chromatrace import LoggingConfig


class SecondService:
    def __init__(self, logging_config: LoggingConfig):
        self.logger = logging_config.get_logger(self.__class__.__name__)
        self.logger.setLevel(logging.ERROR)
    
    async def do_something(self):
        self.logger.debug("Check something in second service")
        self.logger.info("Doing something in second service")
        self.logger.error("Something went wrong in second service")


class ExampleService:
    def __init__(self, logging_config: LoggingConfig, second_service: SecondService):
        self.logger = logging_config.get_logger(self.__class__.__name__)
        self.second_service = second_service
        self.logger.setLevel(logging.ERROR)
    
    async def do_something(self):
        self.logger.debug("Check something")
        self.logger.info("Doing something")
        self.logger.error("Something went wrong")
        await self.second_service.do_something()




from .fastapi import RequestIdMiddleware  # noqa
from .logging_config import LoggingConfig  # noqa
from .logging_settings import LoggingSettings  # noqa
from .tracer import RequestIdContext, trace_id_ctx  # noqa
from .tracer import tracer as trace  # noqa



import asyncio
import contextvars
import functools
import logging
import uuid
from typing import Optional

trace_id_ctx = contextvars.ContextVar("trace_id", default=None)

class TraceContext:
    def __init__(self, trace_id: Optional[str] = None):
        self.trace_id = trace_id or f"T-{str(uuid.uuid4())[:8]}"
        self.token = None

    def __enter__(self):
        self.token = trace_id_ctx.set(self.trace_id)
        return self
    
    def __exit__(self, *args):
        trace_id_ctx.reset(self.token)

    async def __aenter__(self):
        self.token = trace_id_ctx.set(self.trace_id)
        return self

    async def __aexit__(self, *args):
        trace_id_ctx.reset(self.token)

def tracer(func):
    @functools.wraps(func)
    async def async_wrapper(*args, **kwargs):
        async with TraceContext():
            return await func(*args, **kwargs)

    @functools.wraps(func)
    def sync_wrapper(*args, **kwargs):
        with TraceContext():
            return func(*args, **kwargs)

    if asyncio.iscoroutinefunction(func):
        return async_wrapper
    return sync_wrapper

class RequestIdContext:
    def __init__(self, request_id: Optional[str] = None):
        self.request_id = request_id or f"R-{str(uuid.uuid4())[:8]}"
        self.token = None

    def __enter__(self):
        self.token = trace_id_ctx.set(self.request_id)
        return self

    def __exit__(self, *args):
        trace_id_ctx.reset(self.token)


class RequestIdFilter(logging.Filter):
    def filter(self, record):
        record.trace_id = trace_id_ctx.get() or "NAN"
        return True


from dependency import container

from chromatrace import LoggingConfig, LoggingSettings

container[LoggingSettings] = LoggingSettings()
container[LoggingConfig] = LoggingConfig(
    container[LoggingSettings], 
    application_level='Development', 
    enable_tracing=True, 
    ignore_nan_trace=True
)